IF !DEF(CONSTANTS_INC)
CONSTANTS_INC SET 1

; Registers
rLY EQU $ff44
rLCDC EQU $ff40
rBGP EQU $ff47
rOBP0 EQU $ff48
rOBP1 EQU $ff49
rP1 EQU $ff00
rWX EQU $ff4b
rWY EQU $ff4a 

rNR10 EQU $ff10
rNR11 EQU $ff11
rNR12 EQU $ff12 
rNR13 EQU $ff13 
rNR14 EQU $ff14 

rNR21 EQU $ff16 
rNR22 EQU $ff17 
rNR23 EQU $ff18 
rNR24 EQU $ff19 

rNR30 EQU $ff1a
rNR31 EQU $ff1b 
rNR32 EQU $ff1c 
rNR33 EQU $ff1d
rNR34 EQU $ff1e 

rNR41 EQU $ff20 
rNR42 EQU $ff21 
rNR43 EQU $ff22 
rNR44 EQU $ff23 

rNR50 EQU $ff24 
rNR51 EQU $ff25 
rNR52 EQU $ff26 

; Other 
WINDOW_X_OFFSET EQU 7 
RAM_ENABLE EQU %00001010
RAM_DISABLE EQU %00000000

; Inputs
BUTTON_START EQU $8 
BUTTON_SELECT EQU $4 
BUTTON_B EQU $2 
BUTTON_A EQU $1 
BUTTON_DOWN EQU $80 
BUTTON_UP EQU $40 
BUTTON_LEFT EQU $20 
BUTTON_RIGHT EQU $10 

; Addess Spaces
TILE_BANK_0 EQU $8000
TILE_BANK_1 EQU $9000
TILE_BANK_SHARED EQU $8800
MAP_0 EQU $9800
MAP_1 EQU $9C00
WAVE_PATTERN_RAM EQU $ff30
ROM_BANK_WRITE_ADDR EQU $2000 
RAM_BANK_WRITE_ADDR EQU $4000
RAM_ENABLE_WRITE_ADDR EQU $0000 

; Game Constants 
MAX_SPEED EQU 4 
PLAYER_HEIGHT EQU 12 
PLAYER_WIDTH EQU 8 

LOAD_LEFT EQU 1 
LOAD_RIGHT EQU 2 
LOAD_TOP EQU 4 
LOAD_BOTTOM EQU 8

LOAD_LEFT_BIT EQU 0 
LOAD_RIGHT_BIT EQU 1 
LOAD_TOP_BIT EQU 2 
LOAD_BOTTOM_BIT EQU 3

LAST_LOAD_X EQU 1 
LAST_LOAD_Y EQU 2 

STATE_MENU EQU 0 
STATE_GAME EQU 1 
STATE_FINALE EQU 2 
STATE_DEATH EQU 3 
STATE_PAUSE EQU 4
STATE_SPLASH EQU 5 

NUMBER_TILES_INDEX EQU 96 
SPECIAL_TILES_INDEX EQU 112
BLANK_TILE_INDEX EQU 95 
NUM_SPECIAL_TILES EQU 16 

MAX_HEARTS EQU 3 
MAX_BUBBLES EQU 255

ITEM_NONE EQU 0
ITEM_HEART EQU 1 
ITEM_BUBBLE EQU 2 
ITEM_SECRET_1 EQU 3 
ITEM_SECRET_2 EQU 4 
ITEM_SECRET_3 EQU 5 
ITEM_FERMATA_RUNE EQU 6
ITEM_BASS_RUNE EQU 7 
ITEM_ALLEGRO_RUNE EQU 8

ENEMY_NONE EQU 0 
ENEMY_SLIME EQU 1 
ENEMY_BIRDY EQU 2 
ENEMY_SHOOTER EQU 3 
ENEMY_SPIKE EQU 4 
ENEMY_PLATFORM EQU 5 

SPECIAL_TILE_SPIKE_UP EQU (SPECIAL_TILES_INDEX + 0)
SPECIAL_TILE_SPIKE_DOWN EQU (SPECIAL_TILES_INDEX + 1)
SPECIAL_TILE_SPRING_UP_1 EQU (SPECIAL_TILES_INDEX + 2)
SPECIAL_TILE_SPRING_UP_2 EQU (SPECIAL_TILES_INDEX + 3)
SPECIAL_TILE_SPRING_DOWN_1 EQU (SPECIAL_TILES_INDEX + 4)
SPECIAL_TILE_SPRING_DOWN_2 EQU (SPECIAL_TILES_INDEX + 5)
SPECIAL_TILE_SPRING_RIGHT_1 EQU (SPECIAL_TILES_INDEX + 6)
SPECIAL_TILE_SPRING_RIGHT_2 EQU (SPECIAL_TILES_INDEX + 7)
SPECIAL_TILE_SPRING_LEFT_1 EQU (SPECIAL_TILES_INDEX + 8)
SPECIAL_TILE_SPRING_LEFT_2 EQU (SPECIAL_TILES_INDEX + 9)
SPECIAL_TILE_DOOR_1 EQU (SPECIAL_TILES_INDEX + 12)
SPECIAL_TILE_DOOR_2 EQU (SPECIAL_TILES_INDEX + 13)
SPECIAL_TILE_SECRET_DOOR_1 EQU (SPECIAL_TILES_INDEX + 14)
SPECIAL_TILE_SECRET_DOOR_2 EQU (SPECIAL_TILES_INDEX + 15)

MAX_ENEMIES EQU 5 
MAX_BULLETS EQU 6 
MAX_ITEMS EQU 6

PLAYER_OBJ_INDEX EQU 0 
ITEM_OBJ_INDEX EQU 4
ENEMY_OBJ_INDEX EQU 10 
BULLET_OBJ_INDEX EQU 30 
STARS_OBJ_INDEX EQU 36 
PULSE_OBJ_INDEX EQU 38 

END_ITEM_LIST EQU 0 
END_ENEMY_LIST EQU 0 

ITEM_TILES EQU 96 
ITEM_TILE_HEART EQU 97 
ITEM_TILE_BUBBLE EQU 98 
ITEM_TILE_STAR EQU 99 
ITEM_TILE_SECRET_1 EQU 100
ITEM_TILE_SECRET_2 EQU 104 
ITEM_TILE_SECRET_3 EQU 108 
ITEM_TILE_FERMATA_RUNE EQU 112
ITEM_TILE_BASS_RUNE EQU 116 
ITEM_TILE_ALLEGRO_RUNE EQU 120 
ITEM_TILE_FERMATA_PULSE EQU 125

ENEMY_TILE_SLIME EQU $10 
ENEMY_TILE_BIRDY EQU $18
ENEMY_TILE_SHOOTER EQU $20 
ENEMY_TILE_SPIKE EQU $30 
ENEMY_TILE_BLACK_SPIKE EQU $34 
ENEMY_TILE_PLATFORM EQU $38
 
MAX_ENEMY_LIST_ENTRIES EQU 20 

SLIME_FLAG_JUMP EQU $01 

BIRDY_FLAG_ONE_WAY EQU $01  
BIRDY_FLAG_BOMB EQU $02 
BIRDY_FLAG_ZIG_ZAG EQU $04 

SHOOTER_FLAG_STEP_YVEL EQU $01 
SHOOTER_FLAG_APPLY_GRAVITY EQU $02

SPIKE_FLAG_VERTICAL EQU $01 
SPIKE_FLAG_STATIONARY EQU $02 
SPIKE_FLAG_BLACK EQU $04 				; DO NOT CHANGE! 
SPIKE_FLAG_CHASE EQU $08

PLATFORM_FLAG_VERTICAL EQU $01 
PLATFORM_FLAG_WAIT EQU $02

MAX_BULLETS_PLAYER EQU 1 
MAX_BULLETS_ENEMY EQU 5 

BULLET_PLAYER_TILE EQU $50 
BULLET_ENEMY_TILE EQU $51 

BASS_CANNON_SPEED EQU $0300

DEATH_STATE_COUNTER_MAX EQU 80 

ENDC
